@@ -1,12 +1,25 @@
+import os
 import socket
 from itertools import chain, imap
-from redis.exceptions import ConnectionError, ResponseError, InvalidResponse
+from redis.exceptions import (
+    RedisError,
+    ConnectionError,
+    ResponseError,
+    InvalidResponse,
+    AuthenticationError
+)
 
 try:
     from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
 
+try:
+    import hiredis
+    hiredis_available = True
+except ImportError:
+    hiredis_available = False
+
 class PythonParser(object):
     "Plain Python parsing class"
     MAX_READ_LENGTH = 1000000
@@ -100,6 +113,10 @@ class PythonParser(object):
 
 class HiredisParser(object):
     "Parser class for connections using Hiredis"
+    def __init__(self):
+        if not hiredis_available:
+            raise RedisError("Hiredis is not installed")
+
     def __del__(self):
         try:
             self.on_disconnect()
@@ -136,17 +153,18 @@ class HiredisParser(object):
             response = self._reader.gets()
         return response
 
-try:
-    import hiredis
+if hiredis_available:
     DefaultParser = HiredisParser
-except ImportError:
+else:
     DefaultParser = PythonParser
 
+
 class Connection(object):
     "Manages TCP communication to and from a Redis server"
     def __init__(self, host='localhost', port=6379, db=0, password=None,
                  socket_timeout=None, encoding='utf-8',
                  encoding_errors='strict', parser_class=DefaultParser):
+        self.pid = os.getpid()
         self.host = host
         self.port = port
         self.db = db
@@ -200,7 +218,7 @@ class Connection(object):
         if self.password:
             self.send_command('AUTH', self.password)
             if self.read_response() != 'OK':
-                raise ConnectionError('Invalid Password')
+                raise AuthenticationError('Invalid Password')
 
         # if a database is specified, switch to it
         if self.db:
@@ -268,6 +286,7 @@ class UnixDomainSocketConnection(Connection):
     def __init__(self, path='', db=0, password=None,
                  socket_timeout=None, encoding='utf-8',
                  encoding_errors='strict', parser_class=DefaultParser):
+        self.pid = os.getpid()
         self.path = path
         self.db = db
         self.password = password
@@ -300,6 +319,7 @@ class ConnectionPool(object):
     "Generic connection pool"
     def __init__(self, connection_class=Connection, max_connections=None,
                  **connection_kwargs):
+        self.pid = os.getpid()
         self.connection_class = connection_class
         self.connection_kwargs = connection_kwargs
         self.max_connections = max_connections or 2**31
@@ -307,8 +327,14 @@ class ConnectionPool(object):
         self._available_connections = []
         self._in_use_connections = set()
 
+    def _checkpid(self):
+        if self.pid != os.getpid():
+            self.disconnect()
+            self.__init__(self.connection_class, self.max_connections, **self.connection_kwargs)
+
     def get_connection(self, command_name, *keys, **options):
         "Get a connection from the pool"
+        self._checkpid()
         try:
             connection = self._available_connections.pop()
         except IndexError:
@@ -325,8 +351,10 @@ class ConnectionPool(object):
 
     def release(self, connection):
         "Releases the connection back to the pool"
-        self._in_use_connections.remove(connection)
-        self._available_connections.append(connection)
+        self._checkpid()
+        if connection.pid == self.pid:
+            self._in_use_connections.remove(connection)
+            self._available_connections.append(connection)
 
     def disconnect(self):
         "Disconnects all connections in the pool"
